
/* Import SuperLib for GameScript */
import("util.superlib", "SuperLib", 18);
Result <- SuperLib.Result;
Log <- SuperLib.Log;
Helper <- SuperLib.Helper;
ScoreList <- SuperLib.ScoreList;
Tile <- SuperLib.Tile;
Direction <- SuperLib.Direction;
Town <- SuperLib.Town;
Industry <- SuperLib.Industry;

import("util.text", "Text", 1);

GOAL_STEP_SIZE <- 10;

class CompanyData
{
	_company = null;
	_goal_id = null;
	_goal_value = null;
	_vehicle_count_average = 0; // average * 100
	_transported = 0;

	constructor(company)
	{
		this._company = company;
		this._goal_id = null;
		this._goal_value = null;
		this._vehicle_count_average = 0;
		this._transported = 0;
	}
}

function CompanyData::CreateGoal()
{
	this._goal_id =
		GSGoal.New(this._company, GSCompany.GetName(this._company) +  ": transport at least " + this._goal_value + " cargo items in average per vehicle during a period of 3 months", GSGoal.GT_NONE, 0);
}

class TestGame extends GSController 
{
	_per_company = null;
	constructor()
	{
		_per_company = [];
		
	}
}

function TestGame::Start()
{
	this.SetupGoals();

	GSLog.Info("Setup done");

	// Wait for the game to start
	this.Sleep(1);

	while (true) {
		this.HandleEvents();
		this.SetupGoals();
		this.ScanGoals();
		this.Sleep(10);
	}
}

function TestGame::HandleEvents()
{
	if(GSEventController.IsEventWaiting())
	{
		local ev = GSEventController.GetNextEvent();

		if(ev == null)
			return;

		local ev_type = ev.GetEventType();
		if(ev_type == GSEvent.ET_WINDOW_WIDGET_CLICK)
		{
		}
	}
}

// Add goals
function TestGame::SetupGoals()
{
	for(local c = GSCompany.COMPANY_FIRST; c <= GSCompany.COMPANY_LAST; c++)
	{
		// has goals already been set up for this company?
		local existing = null;
		local existing_idx = 0;
		foreach(company_data in this._per_company)
		{
			if(company_data._company == c)
			{
				existing = company_data;
				break;
			}
			existing_idx++;
		}

		// does the company exist in the game
		if(GSCompany.ResolveCompanyID(c) == GSCompany.COMPANY_INVALID)
		{
			if(existing != null)
			{
				// Remove data for no longer existing company
				this._per_company.remove(existing_idx);
			}
			continue;
		}
	
		// If the company can be resolved and exist (goals has been setup) => don't do anything
		if(existing != null) continue;

		// Company goals has not yet been setup for this company
		local company_data = CompanyData(c);
		company_data._goal_value = GOAL_STEP_SIZE;
		company_data.CreateGoal();

		this._per_company.append(company_data);
	}
}

// Has any goals been fulfilled?
function TestGame::ScanGoals()
{
	foreach(company_data in this._per_company)
	{
		local delivered = GSCompany.GetQuarterlyCargoDelivered(company_data._company, GSCompany.CURRENT_QUARTER + 1);
		local veh_count = GSVehicleList().Count();

		company_data._vehicle_count_average = (company_data._vehicle_count_average * 5 + veh_count * 100) / 6;

		local cargo_per_veh = company_data._vehicle_count_average == 0? 0 :
			delivered * 100 / company_data._vehicle_count_average;

		if(GSController.GetSetting("log_level") > 1)
		{
			GSLog.Info("Cargo/vehicle for " + GSCompany.GetName(company_data._company) + ": " + cargo_per_veh);
		}

		local hq_tile = GSCompany.GetCompanyHQ(company_data._company);
		if(GSMap.IsValidTile(hq_tile))
		{
			Helper.SetSign(hq_tile, "Score: " + (company_data._goal_value - GOAL_STEP_SIZE), true);
			Helper.SetSign(hq_tile + 1, Text(GSText.STR_SCORE, (company_data._goal_value - GOAL_STEP_SIZE).tostring()), true);
		}

		//GSLog.Info(GSText(GSText.STR_TEST));
		//GSLog.Info(Text(GSText.STR_SCORE, "2"));
		

		if(cargo_per_veh > company_data._goal_value)
		{
			GSGoal.Remove(company_data._goal_id);
			company_data._goal_value += GOAL_STEP_SIZE;
			company_data.CreateGoal();
			GSLog.Info(GSCompany.GetName(company_data._company) + " completed its goal");
		}
	}
}
